"""Assessment resource implementation for the MontyCloud DAY2 SDK."""

from typing import Optional

from day2.client.base import BaseClient
from day2.models.assessment import (
    AnswerQuestionInput,
    AnswerQuestionOutput,
    CreateAssessmentInput,
    CreateAssessmentOutput,
    GetAssessmentOutput,
    GetQuestionOutput,
    ListAssessmentsOutput,
    ListQuestionsOutput,
)
from day2.session import Session


class AssessmentClient(BaseClient):
    """Client for interacting with the Assessment service."""

    def __init__(self, session: Session) -> None:
        """Initialize a new AssessmentClient.

        Args:
            session: MontyCloud session.
        """
        super().__init__(session, "assessment")

    def list_assessments(
        self,
        tenant_id: str,
        status: str,
        keyword: Optional[str] = None,
        page_number: int = 1,
        page_size: int = 10,
    ) -> ListAssessmentsOutput:
        """List assessments in a tenant.

        Args:
            tenant_id: The ID of the tenant to list assessments for.
            status: Status to filter assessments by. Must be one of ["PENDING", "COMPLETED"].
            keyword: Optional keyword to filter assessments by name or description.
            page_number: The page number to retrieve (default: 1).
            page_size: The number of assessments per page (default: 10).

        Returns:
            ListAssessmentsOutput: Object containing list of assessments and pagination info.

        Raises:
            ClientError: If the request is invalid.
            ServerError: If an internal server error occurs.
            AuthenticationError: If authentication fails.

        Examples:
            >>> session = Session(api_key="your-api-key")
            >>> client = session.assessment
            >>> response = client.list_assessments(tenant_id="tenant-123", status="PENDING", page_number=1, page_size=10)
            >>> for assessment in response.assessments:
            ...     print(f"{assessment.id}: {assessment.name}")
        """
        params = {
            "Status": status,
            "PageNumber": page_number,
            "PageSize": page_size,
        }

        if keyword:
            params["Keyword"] = keyword

        response = self._make_request(
            "GET", f"tenants/{tenant_id}/assessments/", params=params
        )
        return ListAssessmentsOutput.model_validate(response)

    def get_assessment(self, tenant_id: str, assessment_id: str) -> GetAssessmentOutput:
        """Get details of a specific assessment.

        Args:
            tenant_id: ID of the tenant the assessment belongs to.
            assessment_id: ID of the assessment to get details for.

        Returns:
            GetAssessmentOutput: Object containing assessment details.

        Raises:
            ResourceNotFoundError: If the assessment does not exist.
            ClientError: If the request is invalid.
            ServerError: If an internal server error occurs.
            AuthenticationError: If authentication fails.

        Examples:
            >>> session = Session(api_key="your-api-key")
            >>> client = session.assessment
            >>> response = client.get_assessment("tenant-123", "assessment-123")
            >>> print(f"Assessment name: {response.name}")
        """
        response = self._make_request(
            "GET", f"tenants/{tenant_id}/assessments/{assessment_id}"
        )
        return GetAssessmentOutput.model_validate(response)

    def create_assessment(
        self, tenant_id: str, data: CreateAssessmentInput
    ) -> CreateAssessmentOutput:
        """Create a new assessment in a tenant.

        Args:
            tenant_id: The ID of the tenant to create the assessment in.
            data: The assessment data to create.

        Returns:
            CreateAssessmentOutput: Object containing the created assessment details.

        Raises:
            ClientError: If the request is invalid.
            ServerError: If an internal server error occurs.
            AuthenticationError: If authentication fails.

        Examples:
            >>> session = Session(api_key="your-api-key")
            >>> client = session.assessment
            >>> data = CreateAssessmentInput(
            ...     name="My Assessment",
            ...     description="Assessment description",
            ...     review_owner="user@example.com",
            ...     scope={"type": "account", "accountId": "123456789012"},
            ... )
            >>> response = client.create_assessment("tenant-123", data)
            >>> print(f"Created assessment with ID: {response.id}")
        """
        response = self._make_request(
            "POST",
            f"tenants/{tenant_id}/assessments/",
            json=data.model_dump(by_alias=True),
        )
        return CreateAssessmentOutput.model_validate(response)

    def list_questions(
        self, tenant_id: str, assessment_id: str, pillar_id: str
    ) -> ListQuestionsOutput:
        """List questions by pillar ID for an assessment in a tenant.

        Args:
            tenant_id: ID of the tenant the assessment belongs to.
            assessment_id: ID of the assessment to get questions for.
            pillar_id: ID of the pillar to get questions for.

        Returns:
            ListQuestionsOutput: Object containing list of questions for the specified pillar.

        Raises:
            ResourceNotFoundError: If the assessment or pillar does not exist.
            ClientError: If the request is invalid.
            ServerError: If an internal server error occurs.
            AuthenticationError: If authentication fails.

        Examples:
            >>> session = Session(api_key="your-api-key")
            >>> client = session.assessment
            >>> response = client.list_questions("tenant-123", "assessment-123", "pillar-123")
            >>> for question in response.questions:
            ...     print(f"{question.id}: {question.title}")
        """
        params = {"PillarId": pillar_id}

        response = self._make_request(
            "GET",
            f"tenants/{tenant_id}/assessments/{assessment_id}/questions",
            params=params,
        )
        return ListQuestionsOutput.model_validate(response)

    def get_question(
        self, tenant_id: str, assessment_id: str, question_id: str
    ) -> GetQuestionOutput:
        """Get details of a specific question for an assessment in a tenant.

        Args:
            tenant_id: ID of the tenant the assessment belongs to.
            assessment_id: ID of the assessment the question belongs to.
            question_id: ID of the question to get details for.

        Returns:
            GetQuestionOutput: Object containing question details.

        Raises:
            ResourceNotFoundError: If the assessment or question does not exist.
            ClientError: If the request is invalid.
            ServerError: If an internal server error occurs.
            AuthenticationError: If authentication fails.

        Examples:
            >>> session = Session(api_key="your-api-key")
            >>> client = session.assessment
            >>> response = client.get_question("tenant-123", "assessment-123", "question-123")
            >>> print(f"Question: {response.title}")
            >>> print(f"Status: {'Answered' if response.is_answered else 'Not answered'}")
        """
        response = self._make_request(
            "GET",
            f"tenants/{tenant_id}/assessments/{assessment_id}/questions/{question_id}",
        )
        # Create the output model and set the id field
        result = GetQuestionOutput.model_validate(response)
        result.id = question_id

        # If PillarName is not in the response but we have a PillarId, try to set it
        if not result.pillar_name and result.pillar_id:
            # Common pillar mappings
            pillar_names = {
                "operational-excellence": "Operational Excellence",
                "security": "Security",
                "reliability": "Reliability",
                "performance-efficiency": "Performance Efficiency",
                "cost-optimization": "Cost Optimization",
                "sustainability": "Sustainability",
            }
            result.pillar_name = pillar_names.get(result.pillar_id, result.pillar_id)

        return result

    def answer_question(
        self,
        tenant_id: str,
        assessment_id: str,
        question_id: str,
        data: AnswerQuestionInput,
    ) -> AnswerQuestionOutput:
        """Answer a specific question for an assessment in a tenant.

        Args:
            tenant_id: ID of the tenant the assessment belongs to.
            assessment_id: ID of the assessment the question belongs to.
            question_id: ID of the question to answer.
            data: The answer data for the question.

        Returns:
            AnswerQuestionOutput: Object containing the status and message from the API.

        Raises:
            ResourceNotFoundError: If the assessment or question does not exist.
            ClientError: If the request is invalid.
            ServerError: If an internal server error occurs.
            AuthenticationError: If authentication fails.

        Examples:
            >>> session = Session(api_key="your-api-key")
            >>> client = session.assessment
            >>> data = AnswerQuestionInput(
            ...     Reason="ARCHITECTURE_CONSTRAINTS",
            ...     ChoiceUpdates={"choice1": {"Status": "SELECTED"}, "choice2": {"Status": "SELECTED"}},
            ...     Notes="Additional notes about the answer",
            ...     IsApplicable=True,
            ...     LensAlias="wellarchitected"
            ... )
            >>> response = client.answer_question("tenant-123", "assessment-123", "question-123", data)
            >>> print(f"Status: {response.status}, Message: {response.message}")
        """
        # The API expects the data directly, not wrapped
        model_data = data.model_dump(by_alias=True)

        response = self._make_request(
            "PUT",
            f"tenants/{tenant_id}/assessments/{assessment_id}/questions/{question_id}",
            json=model_data,
        )

        # Create the output model with the success response
        result = AnswerQuestionOutput.model_validate(response)

        # Set the question ID from the parameter
        result.id = question_id

        return result
